
' WATERFALL (1999)
' 
'
'                                    Copyright (C) 1999, SEAV Softwares
'
'                                    This program written by Eugene Villar
'                                    a.k.a SEAV (evillar@iname.com).
'
'                                    Visit my web site at
'                                    http://seav.findhere.com
'
' 
'
' This program draws a waterfall effect using a particle engine with motion
' blurring. Done in pure QB (so expect it to be a bit slow).
'
' Controls:
'   +/-: Increase or decrease the number of particles by 50. [50-4000]
'   Tab: Toggle on/off framerate & particles status display.
'   Esc: Quit


DECLARE SUB PAL (Col%, R%, G%, B%)
DEFINT A-Z
RANDOMIZE TIMER

' Define water particles
TYPE particle
   x AS SINGLE
   y AS SINGLE
   dx AS SINGLE
   dy AS SINGLE
END TYPE
REDIM Water(1 TO 4000) AS particle

' Initialize particles
Particles = 2000
FOR I = 1 TO Particles
   Water(I).x = RND * 70
   Water(I).y = RND * 149
   Water(I).dx = RND + .5
   Water(I).dy = RND / 10
   Water(I + 2000).x = RND * 5 + 5
   Water(I + 2000).y = RND * 5 + 5
   Water(I + 2000).dx = RND + .5
   Water(I + 2000).dy = RND / 10
NEXT

' Define Sprite buffer
REDIM Buffer(5473)                     ' Sprite array
Buffer(0) = 576                        ' Set sprite dimensions
Buffer(1) = 152

DEF SEG = VARSEG(Buffer(0))            ' Fill sprite with color 4
FOR P = 4 TO 10947
   POKE P, 4
NEXT

' Define palette and screen items
SCREEN 13
FOR I = 0 TO 127                       ' Blue-white gradient
   PAL I, I \ 4, I \ 3, I \ 2
   PAL I + 128, I \ 6 + 31, I \ 6 + 41, 63
NEXT
FOR I = 1 TO 48                        ' Background design
   LINE (0, I + 100)-(319, I + 100), I * 2
NEXT
FOR I = -1 TO 7                        ' Program title
   PSET (I * 54 - 10, 141), 80: DRAW "c30bd5u3rd4r2u4rd4r2u4rnd3be2bd3ddfu4r3d4nl3ru4be2brd2nld3fnru4nru2br3bd3ddfnr3u4r3ddnl2rube2bubd2d4ru4r2flbe3bd2rnr2d4ru6gerbr2bd3ddfu4r3d4nl3ru4be2d6ru6br2d6ru6br2"
   PSET (I * 54 - 11, 140), 80: DRAW "c255bd5u3rd4r2u4rd4r2u4rnd3be2bd3ddfu4r3d4nl3ru4be2brd2nld3fnru4nru2br3bd3ddfnr3u4r3ddnl2rube2bubd2d4ru4r2flbe3bd2rnr2d4ru6gerbr2bd3ddfu4r3d4nl3ru4be2d6ru6br2d6ru6br2"
NEXT
LINE (70, 24)-STEP(73, 153), 32, B     ' Frames
LINE (71, 25)-STEP(71, 151), 4, B

' Define look-up tables
REDIM Average(1275)                    ' Pixel averages
REDIM Rand(1000)                       ' Random numbers
REDIM Col(151)                         ' Color values
REDIM YOffset(151)                     ' Sprite memory offset
FOR I = 6 TO 1275
   Average(I) = I / 5.4
NEXT
FOR I = 0 TO 1000
   Rand(I) = RND * 5
NEXT
FOR I = 0 TO 151
   Col(I) = (150 - I) / 150 * 247 + 3
   YOffset(I) = I * 72 + 77
NEXT


' Start loop
OldTime! = TIMER
DO
   Frames& = Frames& + 1
   
   ' Particle Engine
   FOR I = 1 TO Particles
      Cols = Col(Water(I).y) + Rand(Frames& MOD 1001)    ' Plot particle
      POKE Water(I).x + YOffset(Water(I).y), Cols

      Water(I).x = Water(I).x + Water(I).dx              ' Update position
      Water(I).y = Water(I).y + Water(I).dy              ' and vectors
      Water(I).dy = Water(I).dy + RND * .35
     
      IF Water(I).y > 149 OR Water(I).x > 69 THEN        ' Initialize new
         Water(I).x = RND * 5 + 5                        ' particle if out of
         Water(I).y = RND * 5 + 5                        ' bounds
         Water(I).dx = RND + .5
         Water(I).dy = RND / 10
      END IF
   NEXT
   
   ' Blur
   FOR P = 76 TO 10875
      Pixel = Average(PEEK(P) + PEEK(P + 1) + PEEK(P - 1) + PEEK(P + 72) + PEEK(P - 72))
      POKE P, Pixel
   NEXT

   ' Draw frame
   PUT (71, 25), Buffer, PSET

   ' Draw status display
   NewTime! = TIMER                                      ' Calculate FPS
   IF NewTime! - OldTime! > 1 THEN
      OldTime! = NewTime!
      FPS = Frames& - OldFrames&
      OldFrames& = Frames&
   END IF
   IF Status THEN                                        ' Display status
      COLOR 128
      LOCATE 21, 20: PRINT LTRIM$(STR$(Particles)) + " Particles  "
      LOCATE 22, 20: PRINT LTRIM$(STR$(FPS)) + " FPS  "
   END IF

   ' Controls
   In$ = INKEY$
   SELECT CASE ASC(In$ + " ")
   CASE 43                                               ' +: Increase
      Particles = Particles + 50                         ' particles
      IF Particles > 4000 THEN Particles = 4000
   CASE 45                                               ' -: Decrease
      Particles = Particles - 50                         ' particles
      IF Particles < 50 THEN Particles = 50
   CASE 9                                                ' Tab:toggle display
      Status = NOT Status
      LOCATE 21, 20: PRINT "              "
      LOCATE 22, 20: PRINT "              "
   END SELECT

LOOP UNTIL In$ = CHR$(27)

' Ending Message
SCREEN 0
WIDTH 80
COLOR 14, 1
PRINT " WATERFALL" + SPACE$(70)
PRINT
COLOR 15, 0
PRINT "Copyright (C) 1999, SEAV Softwares"
PRINT
COLOR 7, 0
PRINT "Created by Eugene Villar (SEAV)"
PRINT "  E-mail: evillar@iname.com"
PRINT "  URL: http://seav.findhere.com"

SUB PAL (Col, R, G, B)
   OUT &H3C8, Col
   OUT &H3C9, R
   OUT &H3C9, G
   OUT &H3C9, B
END SUB

